<?php

namespace Litecms\Currency\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litecms\Currency\Forms\Currency as CurrencyForm;
use Litecms\Currency\Http\Requests\CurrencyResourceRequest;
use Litecms\Currency\Http\Resources\CurrencyResource;
use Litecms\Currency\Http\Resources\CurrenciesCollection;
use Litecms\Currency\Models\Currency;
use Litecms\Currency\Actions\CurrencyAction;
use Litecms\Currency\Actions\CurrencyActions;


/**
 * Resource controller class for currency.
 */
class CurrencyResourceController extends BaseController
{

    /**
     * Initialize currency resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = CurrencyForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('litecms.currency.modules'), 'currency', guard_url('currency'));
            return $next($request);
        });
    }

    /**
     * Display a list of currency.
     *
     * @return Response
     */
    public function index(CurrencyResourceRequest $request)
    {
        $request = $request->all();
        $page = CurrencyActions::run('paginate', $request);

        $data = new CurrenciesCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('currency::currency.names'))
            ->view('currency::currency.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display currency.
     *
     * @param Request $request
     * @param Model   $currency
     *
     * @return Response
     */
    public function show(CurrencyResourceRequest $request, Currency $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new CurrencyResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('currency::currency.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('currency::currency.show')
            ->output();
    }

    /**
     * Show the form for creating a new currency.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(CurrencyResourceRequest $request, Currency $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new CurrencyResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('currency::currency.name'))
            ->view('currency::currency.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new currency.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(CurrencyResourceRequest $request, Currency $model)
    {
        try {
            $request = $request->all();
            $model = CurrencyAction::run('store', $model, $request);
            $data = new CurrencyResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('currency::currency.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('currency/currency/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/currency/currency'))
                ->redirect();
        }

    }

    /**
     * Show currency for editing.
     *
     * @param Request $request
     * @param Model   $currency
     *
     * @return Response
     */
    public function edit(CurrencyResourceRequest $request, Currency $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new CurrencyResource($model);
        // return view('currency::currency.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('currency::currency.name'))
            ->view('currency::currency.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the currency.
     *
     * @param Request $request
     * @param Model   $currency
     *
     * @return Response
     */
    public function update(CurrencyResourceRequest $request, Currency $model)
    {
        try {
            $request = $request->all();
            $model = CurrencyAction::run('update', $model, $request);
            $data = new CurrencyResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('currency::currency.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('currency/currency/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('currency/currency/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the currency.
     *
     * @param Model   $currency
     *
     * @return Response
     */
    public function destroy(CurrencyResourceRequest $request, Currency $model)
    {
        try {

            $request = $request->all();
            $model = CurrencyAction::run('destroy', $model, $request);
            $data = new CurrencyResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('currency::currency.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('currency/currency/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('currency/currency/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
